9.1 Основы лайв-кодинга

# Лайв-кодинг

Теперь мы узнали достаточно, чтобы начать по-настоящему развлекаться. В этом разделе мы будем черпать из всех предыдущих разделов и узнаем, как вы сможете делать свои музыкальные композиции живыми и превращать их в выступления. Для этого нам понадобится 3 основных компонента:

* Умение писать код, который создаёт звук - ЕСТЬ!
* Умение писать функции - ЕСТЬ!
* Умение использовать (именовать) потоки - ЕСТЬ!

Чудненько, давайте начнём. Будем вживую кодировать наши первые звуки. Начнем с простого. Сначала нам нужна функция, содержащая код, который мы хотим проиграть. Ещё мы хотим зациклить вызовы этой функции в отдельном потоке:

```
define :my_loop do
  play 50
  sleep 1
end
in_thread(name: :looper) do
  loop do
    my_loop
  end
end
```

Если это выглядит для вас довольно сложным, вернитесь и перечитайте разделы о функциях и потоках. Это не сложно, если раньше вы уже прокручивал эти вещи у себя в голове.

То, что мы здесь имеем - это определение функции, которая всего лишь играет ноту 50 и немного ждёт. Затем мы задали именованный поток с именем `:looper`, который бесконечно выполняет цикл с функцией `my_loop`.

Если вы выполните этот код, то услышите ноту 50 повторяющуюся снова и снова...

## Изменяем это

Итак, вот где начинается веселье. В то время, как код *по-прежнему выполняется*, изменим 50 на другое число, скажем, на 55, и снова нажмём кнопку "Выполнить". Ого! Всё изменилось! Вживую!

Это не добавило новый звуковой слой, так как мы используем именованные потоки выполнения, которые допускают только один поток для каждого имени. Кроме того, звук изменился, потому что мы *переопределили* функцию. Мы дали `:my_loop` новое определение. Когда поток `:looper` повторял цикл, он просто вызвал новое определение.

Попробуйте изменить это снова - измените ноту, измените время сна. Как на счёт добавления инструкции `use_synth`? Например, измените код на:

```
define :my_loop do
  use_synth :tb303
  play 50, release: 0.3
  sleep 0.25
end
```

Сейчас это звучит довольно интересно, но мы можем оживить это ещё больше. Вместо того, чтобы играть одну и ту же ноту снова и снова, попробуем сыграть аккорд:

```
define :my_loop do
  use_synth :tb303
  play chord(:e3, :minor), release: 0.3
  sleep 0.5
end
```

Как на счёт воспроизведения случайных нот из аккорда:

```
define :my_loop do
  use_synth :tb303
  play choose(chord(:e3, :minor)), release: 0.3
  sleep 0.25
end
```

Или использования случайных значений частоты среза:

```
define :my_loop do
  use_synth :tb303
  play choose(chord(:e3, :minor)), release: 0.2, cutoff: rrand(60, 130)
  sleep 0.25
end
```

Наконец, добавим немного барабанов:

```
define :my_loop do
  use_synth :tb303
  sample :drum_bass_hard, rate: rrand(0.5, 2)
  play choose(chord(:e3, :minor)), release: 0.2, cutoff: rrand(60, 130)
  sleep 0.25
end
```

Теперь это становится увлекательным!

Однако, прежде чем вы начнёте живое кодирование с функциями и потоками, остановите то, что вы сейчас делаете, и прочтите следующий раздел `live_loop`, который изменит манеру вашего кодирования в Sonic Pi навсегда...
